/**
 * Fork to work persitently.
 * @memberof module:apeman/util/clustering
 * @constructor PersistForker
 * @param {string} base - Base directory path.
 */

"use strict";

var u = require('../../lib'),
    EventEmitter = u.core.events.EventEmitter,
    inherits = u.core.util.inherits,
    copy = u.object.copy,
    doNothing = u.doNothing,
    ClusterHelper = require('./cluster_helper');

/** @lends PersistForker */
function PersistForker(basedir) {
    var s = this;
    EventEmitter.call(s);
    s.helper = new clustering.ClusterHelper(basedir);
}

inherits(PersistForker, EventEmitter);

copy(
    /** @lends PersistForker.prototype */
    {
        shouldStopAll: false,
        bindMaster: function (cluster, process) {
            var s = this,
                h = s.helper;

            var numberOfWorkers = h.numberOfCPUs();
            h.forkClusterWorkers(cluster, numberOfWorkers, function (workers) {
                h.writeStatus({
                    pid: process.pid,
                    numberOfWorkers: numberOfWorkers,
                    startAt: new Date
                });
                workers.forEach(function (worker) {
                    s.emit('fork', worker.process.pid);
                });
            });

            h.bindReforkClusterWorker(cluster, function shouldRefork(worker) {
                s.emit('unfork', worker.process.pid);
                var refork = !s.shouldStopAll;
                if (refork) {
                    s.emit('refork', worker.process.pid);
                }
                return  refork;
            });

            process.on('SIGUSR2', function () {
                s.shouldStopAll = true;
                h.sendToAllClusterWorkers(cluster, {
                    cmd: 'stop'
                });
                s.emit('stop', {});
            });

            s.emit('start');
        },
        bindWorker: function (cluster, process) {
            var s = this;
            s.emit('start');
            process.on('exit', function () {

            });
            process.on('message', function (msg) {
                var shouldStop = msg.cmd && msg.cmd == 'stop';
                if (shouldStop) {
                    console.log("\tWork stopped with pid:", process.pid);
                    s.emit('stop');
                    process.exit();
                }
            });
        }
    },
    PersistForker.prototype
);


module.exports = PersistForker;